#ifndef PREFERENCES_H
#define PREFERENCES_H

#include <QtGui/QtGui>

class Preferences : public QDialog
{
  Q_OBJECT
  
public:
  Preferences(QWidget *parent=0);
  ~Preferences();
  
  void load();
  void save();
  
  // ----- Defaults -----
  static void getImageDefaults(QString &file,
                               QString &title,
                               QString &id,
                               int &counter);
  static void setImageDefaults(const QString &file,
                               const QString &title,
                               const QString &id,
                               int counter);
  static void getNextImageName(QString &file,QString &title,QString &id);
  
  static void getEmulatorDefaults(QString &app,
                                  QString &mountArgs,
                                  QString &runArgs);
  static void setEmulatorDefaults(const QString &app,
                                  const QString &mountArgs,
                                  const QString &runArgs);
  
  static void setFontDefaults(bool shifted,bool fixed);
  static void getFontDefaults(bool &shifted,bool &fixed);
  static QFont getDefaultFont();
  static QFont getFont(bool shifted,bool fixed);
  
  static void setSeparatorDefaults(const QStringList &templates);
  static void getSeparatorDefaults(QStringList &templates);

protected slots:
  void onAddSeparator();
  void onDelSeparator();

protected:
  QLineEdit *m_imgFileEdit;
  QLineEdit *m_imgTitleEdit;
  QLineEdit *m_imgIdEdit;
  QLineEdit *m_imgCounterEdit;
  QCheckBox *m_fontShiftedCheck;
  QCheckBox *m_fontFixedCheck;
  QListView *m_separatorView;
  QStringListModel *m_separatorModel;
  QLineEdit *m_emuAppEdit;
  QLineEdit *m_emuMountArgsEdit;
  QLineEdit *m_emuRunArgsEdit;

  QWidget *createImageGroup(QWidget *parent);
  QWidget *createFontGroup(QWidget *parent);
  QWidget *createSeparatorGroup(QWidget *parent);
  QWidget *createEmulatorGroup(QWidget *parent);

  static bool insertCounter(const QString &pattern,int counter,QString &result);
};

#endif
